/** @file modbusclient.h
* @brief client implementation of common MODBUS functions
*
* @author S.Nahas
* @date Feb 27, 2009
*
* @version  $LastChangedRevision: 3021 $
* @version $LastChangedDate: 2008-06-11 10:16:10 +0200 (Wed, 11 Jun 2008) $
*
*
* @par LICENCE
* @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; either version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

   You shoud also find the complete GPL in the COPYING file
   accompanying this source code.
   @endverbatim
*
*
*/

#ifdef WIN32
#include "../../Interface_Library/aes.h"
#else
#include "aes.h"
#endif

#include <modbusclient.h>
#include <addidata_time.h>

#define MFMARK(__fn) printf("calling MODBUS function %s\n",__fn)

//---------------------------------------------------------------------------------
int sample_MXCommon__GetModuleType(struct modbus * modbus)
{
	struct MXCommon__GetModuleType_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__GetModuleType");
	ret = modbus_call_MXCommon__GetModuleType(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__GetModuleType") )
	{
		printf("module type: %s\n",Result.str);
		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MXCommon__GetModuleTypeEx(struct modbus * modbus)
{
	struct MXCommon__GetModuleTypeEx_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__GetModuleTypeEx");
	ret = modbus_call_MXCommon__GetModuleTypeEx(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__GetModuleTypeEx") )
	{
		printf("module type: %s\n",Result.str);
		return 0;
	}
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MXCommon__GetTime(struct modbus * modbus)
{
	struct MXCommon__GetTime_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__GetTime");
	ret = modbus_call_MXCommon__GetTime(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__GetTime") )
	{
		addidata_time_display ( (struct timeval *) (&Result), "current time on module: ");
		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MXCommon__GetTimeEx(struct modbus * modbus)
{
	struct MXCommon__GetTimeEx_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__GetTimeEx");
	ret = modbus_call_MXCommon__GetTimeEx(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__GetTimeEx") )
	{
		addidata_time_display ( (struct timeval *) (&Result), "current time on module: ");
		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
unsigned char pb_Key[32] = {0x1,0x2,0x03,0x4,0x5,0x6,0x7,0x8,0x9,0x10,0x11,0x12,0x13,0x14,0x15,0x16,
		0x17,0x18,0x19,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x30,0x31,0x32};

unsigned char pb_IV[16] = {0x1,0x2,0x03,0x4,0x5,0x6,0x7,0x8,0x9,0x10,0x11,0x12,0x13,0x14,0x15,0x16};
//---------------------------------------------------------------------------------
static int TestCustomerID(unsigned char *bCryptedValue,unsigned char *bValue,int i_Length)
{
	int i_ReturnValue = -1;
	str_AES s_AES;
	unsigned char pb_EncryptedValue[16];


	AES_SetKey(&s_AES,pb_Key,pb_IV);
	AES_ConvertKey(&s_AES);
	AES_CBC_Decrypt(&s_AES,bCryptedValue,pb_EncryptedValue,i_Length);

	if(memcmp(bValue,pb_EncryptedValue,i_Length) == 0)
		i_ReturnValue = 0;

	return (i_ReturnValue);

}
//---------------------------------------------------------------------------------
int sample_MXCommon__TestCustomerID(struct modbus * modbus)
{
	struct MXCommon__TestCustomerID_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__TestCustomerID");
	ret = modbus_call_MXCommon__TestCustomerID(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__TestCustomerID") )
	{
		if (TestCustomerID(Result.bCryptedValueArray,Result.bValueArray,16) == 0)
			printf(" CUSTOMER ID OK \n");
		else
			printf(" WRONG CUSTOMER ID \n");

		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MXCommon__TestCustomerIDEx(struct modbus * modbus)
{
	struct MXCommon__TestCustomerIDEx_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	MFMARK("MXCommon__TestCustomerIDEx");
	ret = modbus_call_MXCommon__TestCustomerIDEx(modbus,&Result);

	if ( !handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__TestCustomerIDEx") )
	{
		if (TestCustomerID(Result.bCryptedValueArray,Result.bValueArray,16) == 0)
			printf(" CUSTOMER ID OK \n");
		else
			printf(" WRONG CUSTOMER ID \n");

		return 0;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int sample_MXCommon__SetHardwareTriggerFilterTime(struct modbus * modbus, uint32_t ulFilterTime)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Reserved = 0;

	MFMARK("MXCommon__SetHardwareTriggerFilterTime");
	ret = modbus_call_MXCommon__SetHardwareTriggerFilterTime(modbus,ulFilterTime,Reserved,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__SetHardwareTriggerFilterTime" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__SetHardwareTriggerFilterTimeEx(struct modbus * modbus, uint32_t ulFilterTime)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Reserved = 0;

	MFMARK("MXCommon__SetHardwareTriggerFilterTimeEx");
	ret = modbus_call_MXCommon__SetHardwareTriggerFilterTimeEx(modbus,ulFilterTime,Reserved,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__SetHardwareTriggerFilterTimeEx" );
}


//---------------------------------------------------------------------------------
int sample_MXCommon__InitAndStartSynchroTimer(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t ulTimeBase;
	uint32_t ulReloadValue;
	uint32_t ulNbrOfCycle;
	uint32_t ulGenerateTriggerMode;
	uint32_t ulOption01;
	uint32_t ulOption02;
	uint32_t ulOption03;
	uint32_t ulOption04;

	//Initialise variables here
	ulTimeBase = 2;
	ulReloadValue = 1;
	ulNbrOfCycle = 0;
	ulGenerateTriggerMode = 0;
	ulOption01 = 0;
	ulOption02 = 0;
	ulOption03 = 0;
	ulOption04 = 0;

	MFMARK("MXCommon__InitAndStartSynchroTimer");
	ret = modbus_call_MXCommon__InitAndStartSynchroTimer(modbus,ulTimeBase,ulReloadValue,ulNbrOfCycle,ulGenerateTriggerMode,ulOption01,ulOption02,ulOption03,ulOption04,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__InitAndStartSynchroTimer" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__InitAndStartSynchroTimerEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t ulTimeBase;
	uint32_t ulReloadValue;
	uint32_t ulNbrOfCycle;
	uint32_t ulGenerateTriggerMode;
	uint32_t ulOption01;
	uint32_t ulOption02;
	uint32_t ulOption03;
	uint32_t ulOption04;

	//Initialise variables here
	ulTimeBase = 2;
	ulReloadValue = 1;
	ulNbrOfCycle = 0;
	ulGenerateTriggerMode = 0;
	ulOption01 = 0;
	ulOption02 = 0;
	ulOption03 = 0;
	ulOption04 = 0;

	MFMARK("MXCommon__InitAndStartSynchroTimerEx");
	ret = modbus_call_MXCommon__InitAndStartSynchroTimerEx(modbus,ulTimeBase,ulReloadValue,ulNbrOfCycle,ulGenerateTriggerMode,ulOption01,ulOption02,ulOption03,ulOption04,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__InitAndStartSynchroTimerEx" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__StopAndReleaseSynchroTimer(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t ulOption01;

	//Initialise variables here
	ulOption01 = 0;

	MFMARK("MXCommon__StopAndReleaseSynchroTimer");
	ret = modbus_call_MXCommon__StopAndReleaseSynchroTimer(modbus,ulOption01,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__StopAndReleaseSynchroTimer" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__StopAndReleaseSynchroTimerEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t ulOption01;

	//Initialise variables here
	ulOption01 = 0;

	MFMARK("MXCommon___StopAndReleaseSynchroTimerEx");
	ret = modbus_call_MXCommon__StopAndReleaseSynchroTimerEx(modbus,ulOption01,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__StopAndReleaseSynchroTimerEx" );
}

//---------------------------------------------------------------------------------
int sample_MXCommon__Reboot(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy;

	//Initialise variables here
	Dummy = 0;

	MFMARK("MXCommon__Reboot");
	ret = modbus_call_MXCommon__Reboot(modbus,Dummy,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__Reboot" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__RebootEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy;

	//Initialise variables here
	Dummy = 0;

	MFMARK("MXCommon__RebootEx");
	ret = modbus_call_MXCommon__RebootEx(modbus,Dummy,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__RebootEx" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__SetCustomerKey(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Initialise variables here

	MFMARK("MXCommon__SetCustomerKey");
	ret = modbus_call_MXCommon__SetCustomerKey(modbus,pb_Key,pb_IV,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__SetCustomerKey" );
}
//---------------------------------------------------------------------------------
int sample_MXCommon__SetCustomerKeyEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Initialise variables here

	MFMARK("MXCommon__SetCustomerKeyEx");
	ret = modbus_call_MXCommon__SetCustomerKeyEx(modbus,pb_Key,pb_IV,&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MXCommon__SetCustomerKeyEx" );
}
