/*
 * main.c
 *
 *  Created on: 29 janv. 2013
 *      Author: rj092
 *
 * Description of the sample:
 * This sample set the state of a digital I/O channel.
 *
 * It takes in parameters two values:
 * - IP_ADDRESS: the network address of your system
 * - STATE: the state to set. 0 is for low and 1 is for high
 *
 * To get the number of digital I/O the sample uses the function soap_call_MSXExxxx__DigitalIOGetNumberOfChannels
 *
 * To set the state of a channel the sample uses the function soap_call_MSXExxxx__DigitalIOWriteChannel
 */

#include <stdlib.h>
#include <stdio.h>

#include <MSXExxxx.nsmap>

static void usage()
{
	printf("./writedigiochannel IP_ADDRESS CHANNEL STATE \n");
	printf("- IP_ADDRESS: IP address of the system.\n");
	printf("- CHANNEL: Index of the channel that you want to set.\n");
	printf("- STATE: state of the channel (0 or 1). \n");
	exit(1);
}

static void soap_cleanup_memory(struct soap * env)
{
	soap_destroy(env);
	soap_end(env);
	soap_free(env);
}

/* entry point */
int main (int argc, char * argv[] )
{
	struct soap * env;
	char soapip[255] = {0};
	unsigned long channel = 0;
	unsigned long state = 0;
	struct MSXExxxx__unsignedLongResponse response;
	struct MSXExxxx__Response defaultresponse;
	int retval = 0;
	unsigned long channelcount = 0;

	/* check user arguments*/
	if ( argc != 4 )
		usage();

	/* construct the soap ip */
	snprintf(soapip, 255, "%s:5555",argv[1]);

	/* get the channel */
	channel = (unsigned long) atoi(argv[2]);

	/* get the state */
	state = (unsigned long) atoi(argv[3]);
	if ( (state != 0) && (state != 1) )
	{
		printf("STATE must be 0 or 1.\n");
		exit(-1);
	}

	/* create SOAP environment */
	if ( (env = soap_new()) == NULL)
	{
		printf("Error while creating SOAP environment.\n");
		exit(1);
	}
	soap_init2(env, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
	env->send_timeout = 5;
	env->recv_timeout = 5;

	/* get the number of available digital I/O channels */
	retval = soap_call_MSXExxxx__DigitalIOGetNumberOfChannels(env, soapip, NULL, 0, &response);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( response.sResponse.iReturnValue != 0 )
	{
		printf("Error while getting the number of digital I/O channels. (iReturnValue: %d, syserrno; %d)", response.sResponse.iReturnValue, response.sResponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}

	/* copy */
	channelcount = response.ulValue;

	/* check the CHANNEL parameter */
	if ( channel >= channelcount )
	{
		printf("CHANNEL is invalid. It is %lu but must be between 0 and %lu.\n", channel, (channelcount - 1));
		soap_cleanup_memory(env);
		exit(-1);
	}

	/* set the output */
	retval = soap_call_MSXExxxx__DigitalIOWriteChannel(env, soapip, NULL, channel, state, 0, &defaultresponse);
	if ( retval != 0 )
	{
		printf("SOAP connection error, or function not available on the system (%d).\n", retval);
		soap_cleanup_memory(env);
		exit(-1);
	}
	if ( defaultresponse.iReturnValue != 0 )
	{
		printf("Error while setting the state of the digital I/O channel. (iReturnValue: %d, syserrno; %d)", defaultresponse.iReturnValue, defaultresponse.syserrno);
		soap_cleanup_memory(env);
		exit(-1);
	}

	printf("Channel %lu is now set to state %lu.\n", channel, state);

	/* release some resources */
	soap_cleanup_memory(env);

	return 0;
}
