using System;
using System.Windows.Forms;
using System.Threading;
using System.Net.Sockets;

namespace AddiData
{
    namespace ADDIControls
    {
        /// <summary>
        /// Add a propriety Index to the ComboBox Element
        /// </summary>
        public class AddiComboBox : ComboBox
        {
            private int i_Index = 0;

            public AddiComboBox()
            {
            }
            public int Index
            {
                set { i_Index = value; }
                get { return i_Index; }
            }
        }
        /// <summary>
        /// Add a propriety Index to the CheckBox Element
        /// </summary>
        public class AddiCheckBox : CheckBox
        {
            private int i_Index = 0;

            public AddiCheckBox()
            {
            }
            public int Index
            {
                set { i_Index = value; }
                get { return i_Index; }
            }
        }

        /// <summary>
        /// Add a propriety Index to the TextBox Element
        /// </summary>
        public class AddiTextBox : TextBox
        {
            private int i_Index = 0;

            public AddiTextBox()
            {
            }
            public int Index
            {
                set { i_Index = value; }
                get { return i_Index; }
            }
        }
    }

    namespace ADDISocket
    {
        public class AsyncClientStream
        {
            public int DataStreamSize = 0;
            public int NbrOfFrames = 0;
            public int SingleFramesSize = 0;

            private const int MAX_STREAM = 1000;
            private AsyncCallback acReadCallBack = null;
            private Socket sClient = null;
            private int WriteStreamIndex = 0;
            private int ReadStreamIndex = 0;
            private int CurrentReceive = 0;
            private bool Initialised = false;
            private Thread ClientThread = null;
            private byte[] CurrentSocketBuffer = null;
            private byte[][] bStreamDataBuffer = new byte[MAX_STREAM][];
            private ManualResetEvent receiveDone = new ManualResetEvent(false);

            private byte[] GetNextStreamBuffer(Socket SocketClient)
            {
                int SaveReceiveBufferSize = 0;

                /* Test if number of frames and frame size initialised */

                if ((NbrOfFrames != 0) && (SingleFramesSize != 0))
                {
                    /* Test if initialised */

                    if (Initialised == false)
                    {
                        /* Streem buffer initialisation */

                        Initialised = true;
                        WriteStreamIndex = 0;
                        ReadStreamIndex = 0;
                        CurrentReceive = 0;
                        DataStreamSize = SingleFramesSize * NbrOfFrames;

                        try
                        {
                            SaveReceiveBufferSize = SocketClient.ReceiveBufferSize;
                            if (SocketClient.ReceiveBufferSize < (DataStreamSize * MAX_STREAM * 2))
                            {
                                SocketClient.ReceiveBufferSize = DataStreamSize * MAX_STREAM * 2;
                            }
                        }
                        catch
                        {
                            SocketClient.ReceiveBufferSize = SaveReceiveBufferSize;
                        }

                        for (int i = 0; i < MAX_STREAM; i++)
                        {
                            bStreamDataBuffer[i] = new byte[DataStreamSize];
                        }
                        return (bStreamDataBuffer[0]);
                    }
                    else
                    {
                        /* Test if no overflow */

                        if (((WriteStreamIndex + 1) % MAX_STREAM) != ReadStreamIndex)
                        {
                            WriteStreamIndex = (WriteStreamIndex + 1) % MAX_STREAM;
                            return (bStreamDataBuffer[WriteStreamIndex]);
                        }
                        else
                        {
                            MessageBox.Show("Sample buffer overflow", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            return (null);
                        }
                    }
                }
                else
                {
                    return (null);
                }
            }

            private void ClientThreadFct()
            {
                int iFrameSize = 0;

                try
                {
                    while ((Initialised == true) && (sClient.Connected) && (CurrentSocketBuffer != null))
                    {
                        iFrameSize = sClient.Receive(CurrentSocketBuffer, CurrentReceive, DataStreamSize - CurrentReceive, SocketFlags.None);

                        if (iFrameSize > 0)
                        {
                            // Save the number of receive datas
                            CurrentReceive = CurrentReceive + iFrameSize;

                            // Test if receive all datas
                            if (CurrentReceive == DataStreamSize)
                            {
                                CurrentReceive = 0;
                                CurrentSocketBuffer = GetNextStreamBuffer(sClient);
                                receiveDone.Set();
                            }
                        }
                        else if (Initialised == true)
                        {
                            MessageBox.Show("MSX-E module have close the socket connection.\nThis stop the sample", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                            Initialised = false;
                            receiveDone.Set();
                            return;
                        }
                        else
                            return;
                    }
                }
                catch (Exception ex)
                {
                    if (Initialised == true)
                    {
                        Initialised = false;
                        receiveDone.Set();
                        MessageBox.Show("Socket reception exception occur : " + ex.Message, "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
            }

            public bool Start(Socket SocketClient)
            {
                CurrentSocketBuffer = this.GetNextStreamBuffer(SocketClient);

                /* Test if no error occur */

                if (CurrentSocketBuffer != null)
                {
                    sClient = SocketClient;
                    sClient.ReceiveTimeout = 0;
                    ClientThread = new Thread(ClientThreadFct);
                    ClientThread.Priority = ThreadPriority.Highest;
                    ClientThread.Start();
                    return (true);
                }
                else
                {
                    return (false);
                }
            }

            public bool StartViaCallback(Socket SocketClient)
            {
                CurrentSocketBuffer = this.GetNextStreamBuffer(SocketClient);

                /* Test if no error occur */

                if (CurrentSocketBuffer != null)
                {
                    acReadCallBack = new AsyncCallback(ReadCallBack);
                    sClient = SocketClient;
                    sClient.ReceiveTimeout = 0;
                    sClient.BeginReceive(CurrentSocketBuffer, 0, DataStreamSize, SocketFlags.None, acReadCallBack, null);
                    return (true);
                }
                else
                {
                    return (false);
                }
            }

            public bool Stoop()
            {
                if (Initialised == true)
                {
                    Initialised = false;
                    sClient.Shutdown(SocketShutdown.Both);
                    receiveDone.Set();
                    return (true);
                }
                else
                {
                    return (false);
                }
            }

            public byte[] ReadData()
            {
                byte[] ReceiveData = null;

                while ((Initialised == true) && (ReceiveData == null))
                {
                    /* Test if any data avaible */

                    if (WriteStreamIndex != ReadStreamIndex)
                    {
                        ReceiveData = bStreamDataBuffer[ReadStreamIndex];
                        ReadStreamIndex = (ReadStreamIndex + 1) % MAX_STREAM;
                    }
                    else
                    {
                        /* Wait datas */
                        receiveDone.WaitOne();
                    }
                }

                return (ReceiveData);
            }

            private void ReadCallBack(IAsyncResult asyn)
            {
                int iFrameSize = 0;

                // Test if data available

                if ((asyn.IsCompleted == true) && (Initialised == true))
                {
                    // Get the number of datas
                    iFrameSize = sClient.EndReceive(asyn);

                    if (iFrameSize <= 0)
                    {
                        MessageBox.Show("MSX-E module have close the socket connection.\nThis stop the sample", "MSX-E sample error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        Initialised = false;
                        receiveDone.Set();
                        return;
                    }

                    // Save the number of receive datas
                    CurrentReceive = CurrentReceive + iFrameSize;

                    // Test if receive all datas
                    if (CurrentReceive == DataStreamSize)
                    {
                        CurrentReceive = 0;
                        CurrentSocketBuffer = GetNextStreamBuffer(sClient);
                        receiveDone.Set();
                    }

                    // Test if can start a new data read
                    if ((Initialised == true) && (CurrentSocketBuffer != null))
                    {
                        if (sClient.Connected)
                            sClient.BeginReceive(CurrentSocketBuffer, CurrentReceive, DataStreamSize - CurrentReceive, SocketFlags.None, acReadCallBack, null);
                    }
                }
            }
        }
    }
}