using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// global::DigitalIO.Properties.Resources.led_on;

namespace DigitalIO
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Graphics component to show the channel id
        /// </summary>
        private Label[] channel_id;

        /// <summary>
        /// Graphics component to show the channel direction
        /// </summary>
        private RadioButton[] channel_direction_in;
        private RadioButton[] channel_direction_out;
        private Panel[] channel_direction;

        /// <summary>
        /// Graphics component to show the io status of the digital IO
        /// </summary>
        private PictureBox[] channel_io;

        /// <summary>
        /// Graphics used to tell that the io is an output
        /// </summary>
        private Panel[] channel_output;

        /// <summary>
        /// Graphics component to show the filter configuration
        /// </summary>
        private CheckBox[] filter_activated;

        /// <summary>
        /// Graphics to separate ports
        /// </summary>
        private Panel[] separation_panel;

        /// <summary>
        /// Contains the current number of channel
        /// </summary>
        private ulong channel_nb = 0;

        /// <summary>
        /// Store the current status of the io
        /// </summary>
        private bool[] io_status;

        /// <summary>
        /// Web reference to the MSExxxx
        /// </summary>
        private MSXExxxx_WebReference.MSXExxxx msxexxxx_soap;

        /// <summary>
        /// Graphics constants
        /// </summary>
        private const int ID_WIDTH = 100;
        private const int ID_HEIGHT = 15;
        private const int ID_LEFT = 10;
        private const int DIRECTION_PANEL_WIDTH = 100;
        private const int DIRECTION_PANEL_HEIGHT = 20;
        private const int DIRECTION_PANEL_LEFT = 160;
        private const int DIRECTION_WIDTH = 50;
        private const int DIRECTION_HEIGHT = 17;
        private const int DIRECTION_IN_LEFT = 0;
        private const int DIRECTION_OUT_LEFT = 50;
        private const int FILTER_ACTIVATED_WIDTH = 20;
        private const int FILTER_ACTIVATED_HEIGHT = 17;
        private const int FILTER_ACTIVATED_LEFT = 320;

        /// <summary>
        /// Constructor of the forms.
        /// Enables to init the graphics elements.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This method is called when the user clicks on the button "Connect"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void B_Connect_Click(object sender, EventArgs e)
        {
            MSXExxxx_WebReference.DefaultResponse response;
            ulong i = 0;
            ulong j = 0;
            int current_y_position = 5;
            int start_y = 0;
            int stop_y = 0;

            // Reset channel nb
            channel_nb = 0;

            // Stop the timer
            digital_io_status_update.Stop();
            start_visualisation_b.Enabled = false;

            // Disable save new conf
            save_new_configuration.Enabled = false;
            save_filter_configuration.Enabled = false;

            msxexxxx_soap = new MSXExxxx_WebReference.MSXExxxx();

            // Check the ip address given by the user
            if (NetworkHelper.isIpCorrect(TB_IPAddress_0.Text, TB_IPAddress_1.Text, TB_IPAddress_2.Text, TB_IPAddress_3.Text) == false)
            {
                MessageBox.Show("The given ip address is wrong. Please correct it and retry");
                return;
            }

            // Init soap connection
            msxexxxx_soap.Url = "http://" + TB_IPAddress_0.Text + "." + TB_IPAddress_1.Text + "." + TB_IPAddress_2.Text + "." + TB_IPAddress_3.Text + ":5555";
            msxexxxx_soap.Timeout = 3000;

            // Get the number of channels
            try
            {
                response = msxexxxx_soap.DigitalIOGetNumberOfChannels(0, out channel_nb);
                if (response.iReturnValue != 0)
                {
                    MessageBox.Show("Unable to get the number of IOs. " + response.syserrno );
                    return;
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Unable to reach system. Check the ip address");
                return;
            }

            // Construct graphics components
            channel_id = new Label[channel_nb];
            channel_direction_in = new RadioButton[channel_nb];
            channel_direction_out = new RadioButton[channel_nb];
            channel_direction = new Panel[channel_nb];
            filter_activated = new CheckBox[channel_nb];
            separation_panel = new Panel[channel_nb / 32];
            channel_io = new PictureBox[channel_nb];
            io_status = new bool[channel_nb];
            channel_output = new Panel[channel_nb];

            // Clear current
            digital_io_info_panel.Controls.Clear();

            // Get informations
            // For each port. A port is a set of 32 channels
            for (i = 0; i < channel_nb / 32; i++)
            {
                ulong inputs = 0;
                ulong outputs = 0;
                ulong directions = 0;
                ulong filter_time = 0;
                ulong filter = 0;

                start_y = current_y_position;

                // Get available direction
                try
                {
                    response = msxexxxx_soap.DigitalIOGetPortAvailableDirections(i, 0, out inputs, out outputs);
                    if (response.iReturnValue != 0)
                    {
                        MessageBox.Show("Unable to get the available port direction. " + response.syserrno);
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }

                // Get current direction configuration
                try
                {
                    response = msxexxxx_soap.DigitalIOGetPortDirections(i, out directions );
                    if (response.iReturnValue != 0)
                    {
                        MessageBox.Show("Unable to get the port direction configuration. " + response.syserrno);
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }

                // Get filter configuration
                try
                {
                    response = msxexxxx_soap.DigitalIOGetInputsFilterConfiguration( i, 0, out filter_time, out filter );
                    if (response.iReturnValue != 0)
                    {
                        MessageBox.Show("Unable to get the filter configuration. " + response.syserrno);
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }

                // Construct the lines and the io status img
                for (j = i * 32; j < (i + 1) * 32; j++)
                {
                    ulong mask = (ulong)((ulong)1 << (int)(j - 32 * i));

                    // Graphics creation
                    channel_id[j] = new Label();
                    channel_id[j].Text = j.ToString();
                    channel_id[j].SetBounds(ID_LEFT, current_y_position, ID_WIDTH, ID_HEIGHT);
                    digital_io_info_panel.Controls.Add(channel_id[j]);

                    channel_direction[j] = new Panel();
                    channel_direction[j].SetBounds(DIRECTION_PANEL_LEFT, current_y_position, DIRECTION_PANEL_WIDTH, DIRECTION_PANEL_HEIGHT);

                    channel_direction_in[j] = new RadioButton();
                    channel_direction_in[j].Text = "In";
                    channel_direction_in[j].SetBounds(DIRECTION_IN_LEFT, 0, DIRECTION_WIDTH, DIRECTION_HEIGHT);
                    channel_direction[j].Controls.Add(channel_direction_in[j]);

                    channel_direction_out[j] = new RadioButton();
                    channel_direction_out[j].Text = "Out";
                    channel_direction_out[j].SetBounds(DIRECTION_OUT_LEFT, 0, DIRECTION_WIDTH, DIRECTION_HEIGHT);
                    channel_direction[j].Controls.Add(channel_direction_out[j]);

                    digital_io_info_panel.Controls.Add(channel_direction[j]);

                    filter_activated[j] = new CheckBox();
                    filter_activated[j].SetBounds(FILTER_ACTIVATED_LEFT, current_y_position, FILTER_ACTIVATED_WIDTH, FILTER_ACTIVATED_HEIGHT);
                    digital_io_info_panel.Controls.Add(filter_activated[j]);
                    
                    // Update information about available direction
                    if ((inputs & mask) != mask)
                    {
                        channel_direction_in[j].Enabled = false;
                    }
                    if ((outputs & mask) != mask)
                    {
                        channel_direction_out[j].Enabled = false;
                    }

                    // Update information about current direction configuration
                    if ((directions & mask) == mask )
                    {
                        channel_direction_out[j].Checked = true;
                    }
                    else
                    {
                        channel_direction_in[j].Checked = true;
                    }

                    // Update information about current filter configuration
                    if ((filter & mask) == mask)
                    {
                        filter_activated[j].Checked = true;
                    }

                    // The io img
                    channel_io[j] = new PictureBox();
                    channel_io[j].Image = global::DigitalIO.Properties.Resources.led_off;
                    channel_io[j].SetBounds((int)(j - (i * 32)) * 32 + 30, (int)i * 25 + 25, 20, 20);
                    if ((directions & mask) == mask)
                    {
                        channel_io[j].Click += new EventHandler(output_img_click);
                        channel_io[j].Cursor = Cursors.Hand;
                        // Also subtitle the channel to say that it is an output
                        channel_output[j] = new Panel();
                        channel_output[j].BackColor = Color.Black;
                        channel_output[j].SetBounds ( (int)(j - (i * 32)) * 32 + 30, (int)i * 25 + 25 + 15, 15, 1 );
                        visualisation_page.Controls.Add(channel_output[j]);
                    }
                    visualisation_page.Controls.Add(channel_io[j]);

                    stop_y = current_y_position;
                    current_y_position += 22;
                }

                filter_input_time_tb.Text = filter_time.ToString();

                // Add a separation panel between the two ports
                if (i != ((channel_nb / 32)-1))
                {
                    separation_panel[i] = new Panel();
                    separation_panel[i].SetBounds(0, current_y_position, digital_io_info_panel.Width, 1);
                    separation_panel[i].BackColor = Color.Black;
                    digital_io_info_panel.Controls.Add(separation_panel[i]);
                }
            }

            // All is ok, enable save configuration
            save_new_configuration.Enabled = true;
            save_filter_configuration.Enabled = true;
            start_visualisation_b.Enabled = true;
            tabControl1.Enabled = true;
        }

        /// <summary>
        /// Occurs when the user clicks on an image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void output_img_click(object sender, EventArgs e)
        {
            if (digital_io_status_update.Enabled)
            {
                ulong i = 0;
                ulong value = 0;
                int syserrno = 0;
                // Get the sender and the value to set
                for (i = 0; i < channel_nb; i++)
                {
                    if (sender == channel_io[i])
                    {
                        if (io_status[i] == true)
                        {
                            value = 0;
                        }
                        else
                        {
                            value = 1;
                        }
                        break;
                    }
                }

                // Write the new status
                try
                {
                    if (msxexxxx_soap.DigitalIOWriteChannel(i, value, 0, out syserrno) != 0)
                    {
                        MessageBox.Show("Error while writing on output" + syserrno.ToString());
                        digital_io_status_update.Stop();
                        stop_visualisation_b.Enabled = true;
                        start_visualisation_b.Enabled = false;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Error while writing on output");
                    digital_io_status_update.Stop();
                    stop_visualisation_b.Enabled = true;
                    start_visualisation_b.Enabled = false;
                }
            }
            else
            {
                MessageBox.Show("Please start the visualisation by clicking on the button start, before trying to change output state");
            }
        }

        /// <summary>
        /// Function call at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            TB_IPAddress_0.Text = Properties.Settings.Default.IP_Field1;
            TB_IPAddress_1.Text = Properties.Settings.Default.IP_Field2;
            TB_IPAddress_2.Text = Properties.Settings.Default.IP_Field3;
            TB_IPAddress_3.Text = Properties.Settings.Default.IP_Field4;
        }

        /// <summary>
        /// Function called when quitting application
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.IP_Field1 = TB_IPAddress_0.Text;
            Properties.Settings.Default.IP_Field2 = TB_IPAddress_1.Text;
            Properties.Settings.Default.IP_Field3 = TB_IPAddress_2.Text;
            Properties.Settings.Default.IP_Field4 = TB_IPAddress_3.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Occurs when the user clicks on the save new configuration button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void save_new_configuration_Click(object sender, EventArgs e)
        {
            ulong i = 0;
            ulong j = 0;
            int syserrno = 0;

            for (i = 0; i < channel_nb / 32; i++)
            {
                // Construct the direction variable
                ulong direction = 0;
                // Construct the filter variable
                ulong filter = 0;
                ulong filter_input_time = 0;

                // Check the filter input time configuration
                try
                {
                    filter_input_time = Convert.ToUInt32(filter_input_time_tb.Text);
                    if (filter_input_time < 100 || filter_input_time % 10 != 0 || filter_input_time > 167772150)
                    {
                        MessageBox.Show("Error, the filter input time " + filter_input_time_tb.Text + " is not correct. [ 10..167772150 multiple of 10 ] ");
                        continue;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Error, the filter input time " + filter_input_time_tb.Text + " is not correct. [ 10..167772150 multiple of 10 ] ");
                    continue;
                }

                for (j = i * 32; j < (i + 1) * 32; j++)
                {
                    ulong mask = (ulong)((ulong)1 << (int)(j - 32 * i));
                    if (channel_direction_out[j].Checked == true)
                    {
                        direction += mask;
                    }
                    if (filter_activated[j].Checked == true)
                    {
                        filter += mask;
                    }
                }
                
                // Set the direction for the port
                try
                {
                    if (  msxexxxx_soap.DigitalIOSetPortDirections(i, direction, 0, out syserrno) != 0 )
                    {
                        MessageBox.Show("Unable to set the direction. " + syserrno );
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }

                // Set the filter activation configuration
                try
                {
                    if (msxexxxx_soap.DigitalIOEnableDisableInputsFilter( i, filter, 0, out syserrno) != 0)
                    {
                        MessageBox.Show("Unable to set the filter enable/disable mode. " + syserrno);
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }

                // Set the filter input time configuration
                try
                {
                    if (msxexxxx_soap.DigitalIOSetInputsFilterTime( filter_input_time,0, out syserrno) != 0)
                    {
                        MessageBox.Show("Unable to set the input filter time. " + syserrno);
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Unable to reach system. Check the ip address");
                    return;
                }
            }

            MessageBox.Show("Configuration saved");
        }

        /// <summary>
        /// Occurs when the user clicks on the save new configuration button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void save_filter_configuration_Click(object sender, EventArgs e)
        {
            // Redirect
            save_new_configuration_Click(sender, e);
        }

        /// <summary>
        /// Called each time the timer ticks
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void digital_io_status_update_Tick(object sender, EventArgs e)
        {
            MSXExxxx_WebReference.DefaultResponse response;
            ulong i = 0;

            for ( i = 0; i < channel_nb; i++ )
            {
                ulong value = 0;
                try
                {
                    response = msxexxxx_soap.DigitalIOReadChannel(i, 0, out value);
                    if (response.iReturnValue != 0)
                    {
                        MessageBox.Show("Error reading digital io state" + response.syserrno );
                        digital_io_status_update.Stop();
                        stop_visualisation_b.Enabled = true;
                        start_visualisation_b.Enabled = false;
                    }

                    if (value == 0 )
                    {
                        channel_io[i].Image = global::DigitalIO.Properties.Resources.led_off;
                        io_status[i] = false;
                    }
                    else
                    {
                        channel_io[i].Image = global::DigitalIO.Properties.Resources.led_on;
                        io_status[i] = true;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Error reading digital io state");
                    digital_io_status_update.Stop();
                    stop_visualisation_b.Enabled = true;
                    start_visualisation_b.Enabled = false;
                }
                // Update
                visualisation_page.Update();
            }
        }

        /// <summary>
        /// Occurs when the user clicks on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void start_visualisation_b_Click(object sender, EventArgs e)
        {
            stop_visualisation_b.Enabled = true;
            start_visualisation_b.Enabled = false;
            digital_io_status_update.Start();
        }

        /// <summary>
        /// Occurs when the user clicks on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stop_visualisation_b_Click(object sender, EventArgs e)
        {
            stop_visualisation_b.Enabled = false;
            start_visualisation_b.Enabled = true;
            digital_io_status_update.Stop();
        }
    }
}